#include "NativeModule.h"

#include <sensrvchannel.h>
#include <sensrvchannelfinder.h>
#include <sensrvdatalistener.h>
#include <sensrvproximitysensor.h>

_LIT(PROXIMITY, "proximity");
_LIT(UNDEFINED, "undefined");
_LIT(AWAY, "away");
_LIT(CLOSE, "close");


class ProximityModule : public NativeModule, public MSensrvDataListener
	{

private:

	enum Functions
		{
		GetFunction
		};

	CSensrvChannel *proximityChannel;
	TInt proximity;
	TRequestStatus *status;

	~ProximityModule()
		{
		delete proximityChannel;
		}

	Runtime::Value getProx()
		{
		Runtime::Value value;
		if (proximityChannel == NULL)
			{
			//alloca channelFinder e ChannelInfoList
			CSensrvChannelFinder* finder = CSensrvChannelFinder::NewLC();
			RSensrvChannelInfoList channelList;
			CleanupClosePushL(channelList);
			
			//Ricerca canale
			TSensrvChannelInfo info;
			info.iChannelType = KSensrvChannelTypeIdProximityMonitor;
			finder->FindChannelsL(channelList, info);
			
			if (channelList.Count() > 0)//trovato prox sensor
				{
				info = channelList[0];
				proximityChannel = CSensrvChannel::NewL(info);
				proximityChannel->OpenChannelL();
				//metti in ascolto sul canale
				proximityChannel->StartDataListeningL(this, 1, 1, 0);
				}
			
			channelList.Close();
			CleanupStack::Pop(&channelList);
			CleanupStack::PopAndDestroy(finder);
			if (proximityChannel == NULL)
				User::Leave(KErrNotSupported);
			}
			value.SetNumber(proximity);
			return value;
		}

protected:
	TInt ExpectedRuntimeVersion()
		{
		return Runtime::VERSION;
		}

	void ConstructL()
		{
		runtime->AddNativeFunctionL(_L("get"), 0, 0, GetFunction);
		runtime->AddConstantL(_L("undefined"),
				TSensrvProximityData::EProximityUndefined);
		runtime->AddConstantL(_L("away"),
				TSensrvProximityData::EProximityIndiscernible);
		runtime->AddConstantL(_L("close"),
				TSensrvProximityData::EProximityDiscernible);
		}

	Runtime::Value ExecuteL(TInt index, Runtime::Value *params,
			TInt paramCount, TRequestStatus &status)
		{

		Runtime::Value result;
		result.SetNull();
		switch (index)
			{
			case GetFunction:
				result = getProx();
				break;
			}
		return result;
		}

	void Cancel()
		{
		if (status != NULL)
			User::RequestComplete(status, KErrCancel);
		}

	// MSensrvDataListener
	void DataReceived(CSensrvChannel& aChannel, TInt count, TInt dataLost)
		{
		TSensrvChannelInfo info = aChannel.GetChannelInfo();
		if (info.iChannelType == KSensrvChannelTypeIdProximityMonitor)
			{
			TSensrvProximityData proximityData;
			TPckg<TSensrvProximityData> proximityPackage(proximityData);
			aChannel.GetData(proximityPackage);
			proximity = proximityData.iProximityState;
			
			//callback 
			if (status != NULL)
				User::RequestComplete(status, KErrNone);
			}
		}

	// MSensrvDataListener
	void DataError(CSensrvChannel& channel, TSensrvErrorSeverity error)
		{
		}

	// MSensrvDataListener
	void GetDataListenerInterfaceL(TUid interfaceUid, TAny *&interface)
		{
		}
	};

//ENTRY POINTS
EXPORT_C NativeModule* NewProximityModuleL()
	{
	return new (ELeave) ProximityModule;
	}

#ifndef EKA2
GLDEF_C TInt E32Dll(TDllReason)
	{return KErrNone;}
#endif

